import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/** DemoButtonModel - demo of <code>JButton</code>'s data model.<p>
*
* Required reading: 
* <a href="http://java.sun.com/products/jfc/tsc/articles/architecture/index.html">
A swing architecture overview:
* The inside story on JFC component design</a> by Amy Fowler
* <p>
*
* This demo explores the "M" in MVC -- the use of
* a dedicated "model" class to hold the data for a component.
* We'll use the <code>JButton</code> class as an example.<p>
*
* As already noted, JFC/Swing components are designed using a modified
* model-view controller (MVC) architecture.  It is "modified" because
* the view and controller are collapsed into a single entity known
* as a UI delegate.  Our focus here is on the model.<p>
*
* Swing components
* use a data model to store their data.  This might sound trivial for
* something like a button.  Afterall, what data are we taking about?  A
* button is simply pressed or not pressed, right?  In fact, even a
* component
* as simple as a button includes quite a bit of "state information" --
* things like whether the button is enabled or disabled, whether
* the mouse pointer is over the button, and so on.  This information
* is not stored in the button object per se,
* but, rather, in the button's data model.<p>
*
* To illustrate this, we create a <code>JButton</code>
* then retrieve the button's data model.  The model is retrieved
* using the <code>getModel</code>
* method, which, according to the Java API, returns a
* <code>ButtonModel</code> object.  Here's the code:<p>
*
* <pre>
*     JButton b;
*     ButtonModel bm;
*     ...
*     b = new JButton("Click Me");
*     bm = b.getModel();
* </pre>
*
* <code>ButtonModel</code> is an
* interface.  It is implemented in the <code>DefaultButtonModel</code>
* class -- the default data model for a <code>JButton</code>.
* This is confirmed as follows:<p>
*
* <pre>
*     System.out.println(bm);
* </pre>
*
* The output is provided by the <code>toString</code> method:<p>
*
* <pre>
*     javax.swing.DefaultButtonModel@4cb029
* </pre>
*
* OK, so we've got the data model for our button.  What can we do with it?
* In our model-view controller discussions, we noted that
* a component's model is required to "respond to queries about
* its state" and "respond to instructions to change its state".  The former
* is implemented via "is" methods, the latter by "set" methods.
* <code>DefaultButtonModel</code> includes several is-methods, including
* <p>
*
* <pre>
*     public boolean isArmed()
*     public boolean isPressed()
*     public boolean isEnabled()
* </pre>
*
* and the following complimentary set-methods:<p>
*
* <pre>
*     public void setArmed(boolean b)
*     public void setPressed(boolean b)
*     public void setEnabled(boolean b)
* </pre>
*
* The "armed" state indicates partial commitment toward pressing
* the button.  The <code>isArmed</code> method returns true if
* the mouse pointer is over the button.
* The "pressed" state means the button has been
* pressed.  The <code>isPressed</code> method returns true if
* primary mouse button is currently down, and it was pushed down
* while the pointer was over
* the button.  When the mouse button is released -- assuming
* the pointer is still over the button -- the button fires
* an action event.<p>
*
* In this demo, a mouse listener and mouse motion listener are
* added to our button.  On each
* mouse event, the button's armed, pressed, and enabled states are
* retrieved using the is-methods
* and dumped to the console.<p>
*
* Additionally, the demo uses a checkbox with an item listener 
* to enable/disable
* the button.  In the <code>itemStateChanged</code> method, we invoke
* either <code>bm.setEnabled(true)</code> or <code>bm.setEnabled(false)</code>
* to enable/disable the button.  It is more common to invoke these methods
* directly on the button object ("<code>b</code>").  The effect
* is exactly the same; namely, to set the "enabled" state of the button as
* stored in its data model.<p>
*
* The button's new status
* is reflected in subsequent output triggered by the mouse events.
* These behaviours are best appreciated by running the program and
* observing the output on the console as the buttons are selected.<p>
*
* Screen snap...<br>
* <center><img src="DemoButtonModel-1.gif"></center><p>
*
* @see <a href="DemoButtonModel.java">source code</a>
* @author Scott MacKenzie, 2009-2011
*/
public class DemoButtonModel
{
   public static void main(String[] args)
   {
      // use look and feel for my system (Win32)
      try {
         UIManager.setLookAndFeel(
            UIManager.getSystemLookAndFeelClassName());
      } catch (Exception e) {}

      DemoButtonModelFrame frame = new DemoButtonModelFrame();
      frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      frame.setTitle("DemoButtonModel");
      frame.pack();

      // position application window in the center of the display
      Dimension d1 = frame.getSize();
      Dimension d2 = Toolkit.getDefaultToolkit().getScreenSize();
      int x = (d2.width - d1.width) / 2;
      int y = (d2.height - d1.height) / 2; 
      frame.setLocation(new Point(x, y));

      frame.setVisible(true);
   }
}

// -----------------------------
// define the application window
// -----------------------------

class DemoButtonModelFrame extends JFrame
implements ActionListener, ItemListener, MouseListener, MouseMotionListener
{
    // the following avoids a "warning" with Java 1.5.0 complier (?)
	static final long serialVersionUID = 42L;

	private JButton b;
   private ButtonModel bm;

   private JCheckBox cb;

   public DemoButtonModelFrame()
   {
      // -------------------------------
      // create and configure components
      // -------------------------------

      b = new JButton("Click Me");
      bm = b.getModel();

      System.out.println(bm);

      cb = new JCheckBox("Disable button");

      // -------------
      // add listeners
      // -------------

      b.addMouseListener(this);
      b.addMouseMotionListener(this);
      b.addActionListener(this);
      cb.addItemListener(this);

      // ------------------
      // arrange components
      // ------------------

      // put components in a panel

      JPanel p = new JPanel(new GridLayout(2, 1, 10, 10));
      p.setBorder(BorderFactory.createEmptyBorder(25, 25, 25, 25));
      p.add(b);
      p.add(cb);

      // make the panel this extended JFrame's content pane

      this.setContentPane(p);
   }

   // -------------------------------
   // implement ActionListener method
   // -------------------------------

   public void actionPerformed(ActionEvent ae)
   {
      System.out.println("BUTTON CLICKED!");
   }

   // -----------------------------------------
   // implement MouseMotionListener methods (2)
   // -----------------------------------------

   public void mouseDragged(MouseEvent me) { outputButtonStatus(); }
   public void mouseMoved(MouseEvent me) { outputButtonStatus(); }

   // -----------------------------------
   // implement MouseListener methods (5)
   // -----------------------------------

   public void mouseClicked(MouseEvent me) { outputButtonStatus(); }
   public void mouseEntered(MouseEvent me) { outputButtonStatus(); }
   public void mouseExited(MouseEvent me) { outputButtonStatus(); }
   public void mousePressed(MouseEvent me) { outputButtonStatus(); }
   public void mouseReleased(MouseEvent me) { outputButtonStatus(); }

   // -----------------------------
   // implement ItemListener method
   // -----------------------------

   public void itemStateChanged(ItemEvent ie)
   {
      if (cb.isSelected())
         bm.setEnabled(false);
      else
         bm.setEnabled(true);
   }

   // -------------
   // other methods
   // -------------

   public void outputButtonStatus()
   {
      System.out.println("Armed=" + bm.isArmed() +
         " Enabled=" + bm.isEnabled() + " Pressed=" + bm.isPressed());      
   }
}
