import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.border.*;

/** DemoTableModel - same as <code>DemoTable</code>, except demonstrating
* the use of <code>JTable</code>'s data model to hold the data.<p> 
*
* As with other JFC/Swing components, the data for a <code>JTable</code>
* object are
* not stored in the component class, per se, but, rather, in a
* dedicated "data model" class.  This is the "M" in the MVC
* (model-view-controller) architecture used for all JFC/Components.<p>
*
* The data model for <code>JTable</code> objects is defined
* in the <code>TableModel</code> interface in the
* <code>javax.swing.table</code> package.  <code>TableModel</code>
* is implemented
* in the <code>AbstractTableModel</code> class.  The default data
* model for <code>JTable</code> is the <code>DefaultTableModel</code>
* class, a subclass of <code>AbstractTableModel</code>.<p>
*
* In this demo, we work with the same data arrays as in
* <code>DemoTable</code>.  We include just one simple change
* to illustrate the use of a data model.  Instead of passing the
* data arrays to the <code>JTable</code> constructor (as in
* <code>DemoTable</code>), we pass them to the <code>DefaultTableModel</code>
* constructor.  The <code>DefaultTableModel</code> object is then
* passed to the <code>JTable</code> constructor:<p>
*
* <pre>
*     DefaultTableModel dtm = new DefaultTableModel(rowData, columnNames);
*     JTable t = new JTable(dtm);
* </pre>
*
* It hardly seems worthwhile using a dedicated demo program that
* includes such a simple change from a previous demo; however,
* the direct use of a component's data model opens up numerous 
* possibilities.  We'll begin to explore these
* in <code>DemoTableModel2</code>.<p>
*
* For further reading, consult the Java API for <code>JTable</code>,
* <code>TableModel</code>, <code>AbstractTableModel</code>, and
* <code>DefaultTableModel</code>, as well as "How to Use Tables"
* in the JFC/Swing tutorial notes.<p>
*
* @see <a href="DemoTableModel.java">source code</a>
* @author Scott MacKenzie, 2009-2011
*/
public class DemoTableModel
{
   public static void main(String[] args)
   {
      // use look and feel for my system (Win32)
      try {
         UIManager.setLookAndFeel(
            UIManager.getSystemLookAndFeelClassName());
      } catch (Exception e) {}

      DemoTableModelFrame frame = new DemoTableModelFrame();
      frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      frame.setTitle("DemoTableModel");
      frame.pack();

      // position application window in the center of the display
      Dimension d1 = frame.getSize();
      Dimension d2 = Toolkit.getDefaultToolkit().getScreenSize();
      int x = (d2.width - d1.width) / 2;
      int y = (d2.height - d1.height) / 2; 
      frame.setLocation(new Point(x, y));

      frame.setVisible(true);
   }
}

class DemoTableModelFrame extends JFrame
{
    // the following avoids a "warning" with Java 1.5.0 complier (?)
	static final long serialVersionUID = 42L;

   public DemoTableModelFrame()
   {
      // ----------------------------------
      // construct and configure components
      // ----------------------------------

      Object columnNames[] = {
         "Candy Bar", "Calories", "Peanut-Free", "Price"
      };

      Object rowData[][] = {
         { "Mr. Small", new Integer(95), new Boolean(false),
            new Double(1.75) },
         { "Kispy Kunch", new Integer(450), new Boolean(false),
            new Double(2.25) },
         { "Kitch Katch", new Integer(400), new Boolean(true),
            new Double(2.75) },
         { "Wunderbloat", new Integer(1300), new Boolean(false),
            new Double(2.65) },
         { "Saramilk", new Integer(295), new Boolean(true),
            new Double(1.25) },
         { "Big Swede", new Integer(300), new Boolean(false),
            new Double(3.10) },
         { "Oh Hank", new Integer(450), new Boolean(false),
            new Double(4.25) },
         { "Eat-Less", new Integer(333), new Boolean(false),
            new Double(2.50) },
         { "Chewbacca", new Integer(456), new Boolean(false),
            new Double(1.85) },
         { "Arrow", new Integer(375), new Boolean(false),
            new Double(1.75) },
         { "Red Rocket", new Integer(300), new Boolean(true),
            new Double(2.00) }
      };   

      DefaultTableModel dtm = new DefaultTableModel(rowData, columnNames);

      JTable t = new JTable(dtm);
      t.setPreferredScrollableViewportSize(new Dimension(300, 100));

      JScrollPane sp = new JScrollPane(t);

      // ------------------
      // arrange components
      // ------------------

      JPanel p = new JPanel(new BorderLayout());
      p.add(sp, "Center");
      p.setBorder(BorderFactory.createEmptyBorder(25, 25, 25, 25));

      this.setContentPane(p);
   }
}
