import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.border.*;

/** DemoTableModel2 - similar to <code>DemoTableModel</code>, except
* using a custom data model.<p>
*
* There are a number of deficiencies in the table in
* <code>DemoTable</code>.  These are
* corrected here by using a custom data model class to hold the table's
* data.<p>
*
* As already noted, the data for a
* <code>JTable</code> are stored in the table's data model
* class.  By default this is an instance of <code>DefaultTableModel</code>,
* a subclass of <code>AbstractTableModel</code> which implements the
* <code>TableModel</code> interface.  Unfortunately, the default
* behaviour of our table is not very appealing.  This demo includes the
* following improvements:<p>
*
* <ul>
* <li>The cells in columns 1 to 3 are non-editable,
* while the cells in column 4 are editable.
* (All columns are editable in the table in <code>DemoTable</code>.)
*
* <li>The data in columns 2 and 4 are right-justified.  These columns
* contain numeric data, so right justification is most appropriate.
* (All cells
* were left-justified in the table in <code>DemoTable</code>.)
*
* <li>The data in the third column are rendered in checkboxes, rather than
* simply appearing as "true" or "false".
* </ul>
*
* These changes are possible by using a custom data model that
* works more closely with the renderer for the cells
* in the table.  Let's examine these changes in detail.<p>
*
* We replace the following line in <code>DemoTableModel</code><p>
*
* <pre>
*     DefaultTableModel dtm = new DefaultTableModel(rowData, columnNames);
* </pre>
*
* with<p>
*
* <pre>
*     CustomTableModel ctm = new CustomTableModel(rowData, columnNames);
*     ctm.setColumnEditable(3, true);
* </pre>
*
* <code>CustomTableModel</code>, defined as an inner class,
* begins with the following signature:<p>
*
* <pre>
*     class CustomTableModel extends AbstractTableModel
* </pre>
*
* Our custom data model is a bit smarter than
* <code>DefaultTableModel</code> (which also extends
* <code>AbstractTableModel</code>).
* Let's examine this by exploring each of the three
* bullets in the list above.<p>
*
* All cells in the table in <code>DemoTable</code> are editible.  Why?
* The <code>TableModel</code> interface defines a method called
* <code>isCellEditable</code> with the following signature:<p>
*
* <pre>
*     public boolean isCellEditable(int rowIndex, int colIndex)
* </pre>
*
* According to the <code>TableModel</code> API, this method should return
* "true if the cell at rowIndex and columnIndex is editable".  This
* method is implemented in <code>AbstractTableModel</code>, and in its
* API we learn that <code>isCellEditable</code>
* always return
* false.  Going one step further,
* we find that
* <code>DefaultTableModel</code> (a subclass of
* <code>AbstractTableModel</code>) overrides this to
* return "true regardless of parameter values".  The effect is that
* all cells are editable by default, and this is the case for the table
* in <code>DemoTable</code>.<p>
*
* Our custom table model improves on this.  It includes a boolean array
* ("<code>editable</code>")
* with as many entries as columns in the table.  The
* <code>isCellEditable</code> method simply returns the value of this
* array at the specified column index.  This, combined with a new method<p>
*
* <pre>
*     public void setColumnEditable(int colArg, boolean editableArg)
* </pre>
*
* provides the means to selectively control which columns in the
* table are editable.  As seen in the code excerpt above,
* we invoke this method on
* our custom table model ("<code>ctm</code>") with arguments
* "<code>3</code>" and "<code>true</code>".  The effect is to make
* the 4th column (at index 3) editable.  (All other columns are non-editable
* by default.)<p>
*
* The points in the second and third bullets above are related, so we'll
* discuss them together.  The <code>TableModel</code> interface specifies
* a method called <code>getColumnClass</code> with the following
* signature:<p>
*
* <pre>
*     public Class getColumnClass(int columnIndex)
* </pre>
*
* According to the API, this method should "return the most specific
* superclass for all the cell values in the column. This is used by the
* <code>JTable</code> to set up a default renderer and editor
* for the column".  In other words, <code>JTable</code> determines
* the class of entries in each column and uses this information to
* control how entries are presented ("rendered") and edited.
* <code>AbstractTableModel</code>'s implementation "returns
* <code>Object.class</code> regardless of columnIndex".
* <code>DefaultTableModel</code> does not override this method, so
* the return value is always <code>Object.class</code>.  The effect
* is that all columns are treated the same.  They are treated as
* strings according to the value returned by the <code>Object</code>
* class' <code>toString</code> method.<p>
*
* Our custom table model improves on this.  We override this method
* as follows:<p>
*
* <pre>
*     public Class getColumnClass(int col)
*     {
*        return getValueAt(0, col).getClass();
*     }
* </pre>
*
* We return the actual class used for objects in each column.  Now,
* what does all this have to do with the last two bullets in the list
* above?  Quite a bit, in fact.  <code>JTable</code> draws its cells
* using a <i>cell renderer</i>.  To determine which renderer to use,
* <code>JTable</code> first determines if a renderer has been
* explicitly installed for a column (we'll do this in
* <code>DemoTableModel3</code>).  If not,
* it queries each column to determine the class of objects in the
* column.  <code>JTable</code> "knows about" (i.e., chooses
* an appropriate renderer for) the following classes:<p>
*
* <ul>
* <li><code>Boolean</code> - rendered with a check box.
* <li><code>Number</code> - rendered by a right-aligned label.
* <li><code>ImageIcon</code> - rendered by a centered label.
* <li><code>Object</code> - rendered by a label that displays
* the object's string value, left-aligned.
* </ul>
*
* Any column whose objects don't fit in one of the top three categories are
* rendered by default according the fourth category.  Note that
* <code>Number</code> is the superclass to classes such as
* <code>Integer</code> and <code>Double</code>, so columns containing
* objects of these classes (i.e., "numbers") are rendererd right-justified.
* <p>
*
* Despite the obvious improvements in the table shown here,
* a few weaknesses remain.  For one, the price of the candy
* bars in the 4th column is poorly presented.  There is no dollar sign
* on the left and trailing zeros are missing (e.g., "<code>$3.10</code>"
* appears as "<code>3.1</code>").  This is purely an artifact of the
* customary treatment of <code>Double</code> -- the class used for
* the price of the candy bars.  Can we do better?
* As well, the 4th column
* in our table is editable, but how will cells respond to changes?
* Can we constrain user edits to ensure that the cells
* contain valid prices?  We'll work on these improvements in
* <code>DemoTableModel3</code>.<p>
*
* Screen snap...<br>
* <center><img src="DemoTableModel2-1.gif"></center><p>
*
* @see <a href="DemoTableModel2.java">source code</a>
* @author Scott MacKenzie, 2009-2011
*/
public class DemoTableModel2
{
   public static void main(String[] args)
   {
      // use look and feel for my system (Win32)
      try {
         UIManager.setLookAndFeel(
            UIManager.getSystemLookAndFeelClassName());
      } catch (Exception e) {}

      DemoTableModel2Frame frame = new DemoTableModel2Frame();
      frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      frame.setTitle("DemoTableModel2");
      frame.pack();

      // position application window in the center of the display
      Dimension d1 = frame.getSize();
      Dimension d2 = Toolkit.getDefaultToolkit().getScreenSize();
      int x = (d2.width - d1.width) / 2;
      int y = (d2.height - d1.height) / 2; 
      frame.setLocation(new Point(x, y));

      frame.setVisible(true);
   }
}

class DemoTableModel2Frame extends JFrame
{
    // the following avoids a "warning" with Java 1.5.0 complier (?)
	static final long serialVersionUID = 42L;

   public DemoTableModel2Frame()
   {
      // ----------------------------------
      // construct and configure components
      // ----------------------------------

      Object columnNames[] = {
         "Candy Bar", "Calories", "Peanut-Free", "Price"
      };

      Object rowData[][] = {
         { "Mr. Small", new Integer(95), new Boolean(false),
            new Double(1.75) },
         { "Kispy Kunch", new Integer(450), new Boolean(false),
            new Double(2.25) },
         { "Kitch Katch", new Integer(400), new Boolean(true),
            new Double(2.75) },
         { "Wunderbloat", new Integer(1300), new Boolean(false),
            new Double(2.65) },
         { "Saramilk", new Integer(295), new Boolean(true),
            new Double(1.25) },
         { "Big Swede", new Integer(300), new Boolean(false),
            new Double(3.10) },
         { "Oh Hank", new Integer(450), new Boolean(false),
            new Double(4.25) },
         { "Eat-Less", new Integer(333), new Boolean(false),
            new Double(2.50) },
         { "Chewbacca", new Integer(456), new Boolean(false),
            new Double(1.85) },
         { "Arrow", new Integer(375), new Boolean(false),
            new Double(1.75) },
         { "Red Rocket", new Integer(300), new Boolean(true),
            new Double(2.00) }
      };   

      CustomTableModel ctm = new CustomTableModel(rowData, columnNames);
      ctm.setColumnEditable(3, true);

      JTable t = new JTable(ctm);
      t.setPreferredScrollableViewportSize(new Dimension(300, 100));

      JScrollPane sp = new JScrollPane(t);

      // ------------------
      // arrange components
      // ------------------

      JPanel p = new JPanel(new BorderLayout());
      p.add(sp, "Center");
      p.setBorder(BorderFactory.createEmptyBorder(25, 25, 25, 25));

      this.setContentPane(p);
   }

   // -----------
   // inner class
   // -----------

   class CustomTableModel extends AbstractTableModel
   {
		 // the following avoids a "warning" with Java 1.5.0 complier (?)
		static final long serialVersionUID = 42L;

      Object[][] rowData;
      Object[] columnNames;

      boolean editable[];

      public CustomTableModel(Object[][] rowData, Object[] columnNames)
      {
         this.rowData = rowData;
         this.columnNames = columnNames;
         editable = new boolean[rowData.length];
      }

      public int getColumnCount()
      {
         return columnNames.length;
      }

      public int getRowCount()
      {
         return rowData.length;
      }

      public String getColumnName(int idx)
      {
         return (String)columnNames[idx];
      }

      public Object getValueAt(int row, int col)
      {
         return rowData[row][col];
      }

      public void setValueAt(Object value, int row, int col)
      {
         rowData[row][col] = value;
         fireTableCellUpdated(row, col);
      }

      public Class getColumnClass(int col)
      {
         return getValueAt(0, col).getClass();
      }

      public void setColumnEditable(int colArg, boolean editableArg)
      {
         editable[colArg] = editableArg;
      }

      public boolean isCellEditable(int row, int col)
      {
         return editable[col];
      } 
   }  
}
